@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2013 Hannu Väisänen (Hannu.Vaisanen@uef.fi)
+# Copyright 2013-2014 Hannu Väisänen (Hannu.Vaisanen@uef.fi)
 # Program to generate old spellings and common spelling mistakes for Voikko lexicon.
 
 # This program is free software; you can redistribute it and/or modify
@@ -35,12 +35,17 @@
 # spellings and spelling errors that do not exist in real life.
 
 # Compiling (in directory suomimalaga):
-# make vvfst-sukija
+# make vvfst-sukija; make vvfst-sukija-install DESTDIR=~/vvfst/voikkodict/
 
-# Testing:
+# Compiling (in directory suomimalaga/vvfst):
+# cd ..; make vvfst-sukija; make vvfst-sukija-install DESTDIR=~/vvfst/voikkodict/; cd vvfst
+
+# Testing (in directory suomimalaga/vvfst):
 # foma -e "read att all-sukija.att" -e "save stack sukija.fst" -e "quit"
-# date; cat ~/Lataukset/koesanat?.txt | flookup -i sukija.fst | gawk 'length($0) > 0' |sort >test.out; date
+# date; cat ~/Lataukset/koesanat?.txt | flookup -i sukija.fst | gawk 'length($0) > 0' | sort >test.out; date
 # diff test.out ~/Lataukset/vvfst-sukija-testi.out | grep '<.*[+][?]' | less
+# diff test.out ~/Lataukset/vvfst-sukija-testi.out | grep '>.*[+][?]' | less
+# cp test.out ~/Lataukset/vvfst-sukija-testi.out
 
 # Style- ja usage-lippujen arvot suoraan Joukahaisesta:
 # grep -A1 '<style>' ../vocabulary/joukahainen.xml|grep flag|sort -u|gawk '{printf "%s,", substr($1,7,length($1)-13)}'
@@ -59,15 +64,17 @@ OPTIONS = generate_lex_common.get_options()
 
 infile = codecs.open (OPTIONS["destdir"] + u"/all.lexc", "r", "UTF-8")
 outfile = codecs.open (OPTIONS["destdir"] + u"/all-sukija.lexc", 'w', 'UTF-8')
+sukijafile = codecs.open (OPTIONS["destdir"] + u"/poikkeavat-sukija.lexc", 'r', 'UTF-8')
 
 C = u"[qwrtpsšdfghjklzžxcvbnm]"  # Consonants.
 V = u"[aeiouüyåäö]"              # Vovels.
-
+A = u"[aä]"
 
 def makeRePattern (wordClass, word):
     u = u"^\\[%s\\]\\[Xp\\].*%s\\[X\\]" % (wordClass, word)
     u = u.replace ('C', C)
     u = u.replace ('V', V)
+    u = u.replace ('A', A)
     return u
 
 
@@ -124,9 +131,14 @@ re_uusio = makeRe (u"Ln", u".Cuusio")
 re_tio   = makeRe (u"Ln", u"[ik]tio") # Traditio, funktio.
 
 re_toninen = makeRe (u"Ll", u".toninen")
+re_iivinen = makeRe (u"Ll", u"Ciivinen")
 
-re_nuolaista = re.compile (u"\\[Lt\\].* Nuolaista_");
-re_rangaista = re.compile (u"\\[Lt\\].* Rangaista_");
+re_nuolaista = re.compile (u"\\[Lt\\].* Nuolaista_", re.UNICODE)
+re_rangaista = re.compile (u"\\[Lt\\].* Rangaista_", re.UNICODE)
+
+re_Xiljoona = re.compile (u"\\A(?:\\[Bc\\]|\\[Sn\\]|@).*(b|m|tr)iljoon", re.UNICODE)
+
+re_eikAs = makeRe (u"Ll", u"eikAs")
 
 
 # Words to be excluded.
@@ -183,60 +195,15 @@ spelling_pattern_list = [
   (re_ottaa2,  u"ö",   u"öi",  u"Ammottaa",   u"Ammottaa"),
 
   (re_toninen, u"toni", u"tooni"),
+  (re_iivinen, u"iivi", u"ivi"),
 
   (re_nuolaista, u"Nuolaista_"),
   (re_rangaista, u"Rangaista_"),
+
+  (re_eikAs, u"eik", u"ehik"),
 ]
 
 
-
-lexicon_kaunis = [
-    u"LEXICON <WC>SukijaKaunis_<A>",
-    u"[Sg][Nm]eid:eid	<WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sg][Nm]eitt:eitt	<WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sg][Nm]ehitt:ehitt	<WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sg][Nm]ehi:ehi   <WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sp][Nm]eit<A>:eit<A>	<WC>LiOlV_<A>	;",
-    u"[Sp][Nm]ehi<A>:ehi<A>	<WC>LiOlV_<A>	;",
-    u"[Sill][Nm]eisii:eisii	<WC>LiOlN_<A>	;",
-    u"[Sill][Nm]ehisi:ehisi	<WC>LiOlN_<A>	;",
-    u"ehi:ehi   Kerronto_sti_<A> ;",
-    u"e:e	<WC>YhteisetMonikonPaikallissijat_<A>	;",
-    u"e:e	<WC>YhteisetMonikonSijat2_<A>	      ;",
-    u"eh:eh	<WC>YhteisetMonikonPaikallissijat_<A>	;",
-    u"eh:eh	<WC>YhteisetMonikonSijat2_<A>	      ;",
-    u"?Laatusana	ehi:ehi	Voittoaste_<A>	;",
-    u"?Laatusana	eh:eh   Yliaste_<A>	;",
-]
-
-lexicon_altis = [
-    u"LEXICON <WC>SukijaAltis_<A>",
-    u"[Sg][Nm]teid:teid	<WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sg][Nm]teitt:teitt	<WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sg][Nm]tehitt:tehitt	<WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sg][Nm]tehi:tehi   <WC>MonikonGenetiiviEnJatko_<A>	;",
-    u"[Sp][Nm]teit<A>:teit<A>	<WC>LiOlV_<A>	;",
-    u"[Sp][Nm]tehi<A>:tehi<A>	<WC>LiOlV_<A>	;",
-    u"[Sill][Nm]teisii:teisii	<WC>LiOlN_<A>	;",
-    u"[Sill][Nm]tehisi:tehisi	<WC>LiOlN_<A>	;",
-    u"tehi:tehi   Kerronto_sti_<A> ;",
-    u"te:te	<WC>YhteisetMonikonPaikallissijat_<A>	;",
-    u"te:te	<WC>YhteisetMonikonSijat2_<A>	      ;",
-    u"teh:teh	<WC>YhteisetMonikonPaikallissijat_<A>	;",
-    u"teh:teh	<WC>YhteisetMonikonSijat2_<A>	      ;",
-    u"?Laatusana	tehi:tehi	Voittoaste_<A>	;",
-    u"?Laatusana	eh:eh   Yliaste_<A>	;",
-#    u"t:t <WC>SukijaKaunis_<A>",
-]
-
-
-def generate (lexicon, word_class, a):
-    p = []
-    for i in lexicon:
-        p.append (i.replace(u"<WC>", word_class).replace(u"<A>", a))
-    return p
-
-
 def word_class (line):
     L = dict ([(u"[Ll]",  u"Laatusana"),
                (u"[Ln]",  u"Nimisana"),
@@ -270,6 +237,19 @@ def write_altis (line, word):
     outfile.write (u"%s %sSukijaAltis_%s ;\n" % (prefix, word_class(line), A))
 
 
+def write_virkkaa (line, word):
+    prefix = line[0:line.find (u" ")]
+    outfile.write (u"%s SukijaVirkkaa_ä ;\n" % (prefix))
+
+def write_paistaa (line, word):
+    prefix = line[0:line.find (u" ")]
+    outfile.write (u"%s SukijaPaistaa_a ;\n" % (prefix))
+
+def write_paahtaa (line, word):
+    prefix = line[0:line.find (u" ")]
+    outfile.write (u"%s SukijaPaahtaa_a ;\n" % (prefix))
+
+
 def generate_from_pattern (line, pattern_list):
     for x in pattern_list:
         if x[0].match(line):
@@ -298,23 +278,34 @@ word_list = [
     (u"ainainen",         (u"ainai",         u"ainahi")),
     (u"aineisto",         (u"aineisto",      u"ainehisto")),
     (u"aksiomi",          (u"aksiom",        u"aksioom")),
+    (u"alamainen",        (u"alamai",        u"alammai")),
+    (u"alimmainen",       (u"alimmai",       u"alimai")),
     (u"alkovi",           (u"alkov",         u"alkoov")),
     (u"ameba",            (u"ameb",          u"ameeb")),
+    (u"amfiteatteri",     (u"amfiteatter",   u"amfiiteaatter", u"amfiteaatter")),
+    (u"apassi",           (u"apass",         u"apash")),
     (u"arsenikki",        (u"arsenik",       u"arseniik")),
     (u"assistentti",      (u"assistent",     u"asistent")),
+    (u"Australia",        (u"Australi",      u"Austraali")),
     (u"barbaari",         (u"barbaar",       u"barbar")),
     (u"beduiini",         (u"beduiin",       u"beduin")),
-    (u"biljoona",         (u"biljoon",       u"biljon")),
+    (u"borssi",           (u"borssi",        u"borsh")),
+    (u"dervissi",         (u"derviss",       u"dervish")),
+    (u"divaani",          (u"divaan",        u"divan")),
     (u"emali",            (u"emal",          u"emalj")),
+    (u"ensimmäinen",      (u"ensimmäi",      u"ensimäi")),
     (u"eteinen",          (u"etei",          u"etehi")),
     (u"evankelinen",      (u"evankeli",      u"evankeeli")),
+    (u"germaani",         (u"germaan",       u"german")),
     (u"haupitsi",         (u"haupits",       u"haubits")),
     (u"hevonen",          (u"hevo",          u"hevoi")),
     (u"humaaninen",       (u"humaani",       u"humani")),
     (u"husaari",          (u"husaar",        u"husar")),
     (u"huumori",          (u"huumor",        u"humor")),
+    (u"inhimillinen",     (u"inhimilli",     u"inhimmilli")),
     (u"inkvisiittori",    (u"inkvisiittor",  u"inkvisitor", u"inkvisiitor")),
     (u"inkvisitio",       (u"inkvisitio",    u"inkvisiitio")),
+    (u"insinööri",        (u"insinöör",      u"insinör")),
     (u"invalidi",         (u"invalid",       u"invaliid")),
     (u"juridinen",        (u"juridi",        u"juriidi")),
     (u"kahdeksainen",        (u"kahdeksai",         u"kaheksai")),
@@ -328,65 +319,113 @@ word_list = [
     (u"kaleeri",        (u"kaleer",      u"kaler")),
     (u"kamari",         (u"kamar",       u"kammar")),
     (u"kameli",         (u"kamel",       u"kameel")),
+    (u"kamiina",        (u"kamiin",      u"kamin")),
     (u"kampanja",       (u"kampanj",     u"kamppanj")),
     (u"kaneli",         (u"kanel",       u"kaneel")),
+    (u"kanuuna",        (u"kanuun",      u"kanun")),
     (u"kaoottinen",     (u"kaootti",     u"kaaootti", u"kaaotti", u"kaotti")),
+    (u"Kaukasia",       (u"Kaukasi",     u"Kaukaasi")),
+    (u"katolilainen",   (u"katolilai",   u"katoolilai")),
+    (u"katolinen",      (u"katoli",      u"katooli")),
     (u"kavaljeeri",     (u"kavaljeer",   u"kavalier", u"kavaljier")),
+    (u"kenraali",       (u"kenraal",     u"kenral")),
+    (u"keskimmäinen",   (u"keskimmäi",   u"keskimäi")),
     (u"kirjoitelma",    (u"kirjoitelm",  u"kirjotelm")),
+    (u"kollegio",       (u"kollegio",    u"kolleegio")),
     (u"koneisto",       (u"koneisto",    u"konehisto")),
     (u"konossementti",  (u"konossement", u"konnossement")),
     (u"kortteeri",      (u"kortteer",    u"kortter", u"kortier", u"korttier")),
-    (u"kraatteri",      (u"kraatter",    u"kraater")),
+    (u"kraatteri",      (u"kraatter",    u"kraater", u"krateer")),
+    (u"kritiikki",      ((u"kritiik",    u"kritik", u"NimisanaTakki_ä", u"NimisanaRisti_ä"),
+                         (u"kritiik",    u"kriitiik"))),
     (u"kulttuuri",      (u"kulttuur",    u"kultuur", "kulttur")),
     (u"kuriiri",        (u"kuriir",      u"kurier")),
+    (u"kurtiini",       (u"kurtiin",     u"kurtin")),
+    (u"kuvernööri",     (u"kuvernöör",   u"kuvernör")),
     (u"leegio",         (u"leegio",      u"legio")),
     (u"legioona",       (u"legioon",     u"legion")),
     (u"legioonalainen", (u"legioonalai", u"legionalai")),
+    (u"lestadiolainen", (u"lestadiolai", u"laestadiolai")),
     (u"liipaisin",      (u"liipaisi",    u"liipasi")),
+    (u"likimmäinen",    (u"likimmäi",    u"likimäi")),
     (u"lordi",          (u"lord",        u"loord")),
+    (u"luterilainen",   (u"luterilai",   u"lutherilai", u"luteerilai")),
+    (u"lähimmäinen",    (u"lähimmäi",    u"lähimäi")),
     (u"majoneesi",      (u"majonees",    u"majonnees")),
+    (u"majuri",         (u"majur",       u"majuur")),
+    (u"mansetti",       (u"manset",      u"manshet")),
     (u"matrikkeli",     (u"matrikkel",   u"matrikel")),
     (u"mieluinen",      (u"mielui",      u"mieluhi")),
-    (u"miljoona",       (u"miljoon",     u"miljon")),
+    (u"minareetti",     (u"minareet",    u"minaret")),
     (u"modeemi",        (u"modeem",      u"modem")),
+    (u"moduuli",        (u"moduul",      u"modul")),
     (u"mosaiikki",      (u"mosaiik",     u"mosaik")),
-    (u"paneeli",        (u"paneel",      u"panel")),
-    (u"paronitar",      (u"paronit",     u"paroonit")),
-    (u"pataljoona",     (u"pataljoon",   u"pataljon")),
-    (u"patriisi",       (u"patriis",     u"patris")),
-    (u"patruuna",       (u"patruun",     u"patrun")),
-    (u"persoona",       (u"persoon",     u"person")),
-    (u"pioneeri",       (u"pioneer",     u"pioner")),
-    (u"pioni",          (u"pion",        u"pioon")),
-    (u"plutoona",       (u"plutoon",     u"pluton")),
-    (u"poliisi",        (u"poliis",      u"polis")),
-    (u"poliitikko",     (u"poliitik",    u"politik", u"poliitiik")),
-    (u"poliittinen",    (u"poliitti",    u"politti", u"poliittii")),
-    (u"politiikka",     (u"politiik",    u"politik", u"poliitiik")),
-    (u"posliini",       (u"posliin",     u"poslin"   u"porsliin", u"porslin")),
-    (u"preettori",      (u"preettor",    u"preetor")),
-    (u"pyramidi",       (u"pyramid",     u"pyramiid")),
-    (u"reettori",       (u"reettor",     u"reetor")),
-    (u"sampanja",       (u"sampanj",     u"samppanj")),
-    (u"sapatti",        (u"sapat",       u"sabat")),
-    (u"sapeli",         (u"sapel",       u"sapeel")),
-    (u"senaatti",       ((u"senaat",     u"senaat", u"NimisanaTatti_a", u"NimisanaRisti_a"),
-                         (u"senaat",     u"senat",  u"NimisanaTatti_a", u"NimisanaRisti_a"))),
-    (u"senaattori",     (u"senaattor",   u"senaator")),
-    (u"serafi",         (u"seraf",       u"seraaf")),
-    (u"shampanja",      (u"shampanj",    u"shamppanj")),
-    (u"sihteeri",       (u"sihteer",     u"sihter", u"sihtier")),
-    (u"sitruuna",       (u"sitruun",     u"sitrun", u"sitroon", u"sitron")),
-    (u"soolo",          (u"soolo",       u"solo")),
-    (u"soopeli",        (u"soopel",      u"sopel", u"soobel")),
-    (u"synagoga",       (u"synagog",     u"synagoog")),
-    (u"teatteri",       (u"teatter",     u"teaater", u"teaatter", u"teater")),
-    (u"temperamentti",  (u"temperament", u"tempperament")),
-    (u"tooga",          (u"toog",        u"tog")),
-    (u"Toscana",        (u"toscan",      u"toskan")),
-    (u"tällainen",      ((u"tällai",     u"tälläi", u"NimiLaatusanaNainen_a", u"NimiLaatusanaNainen_ä"), )),
-    (u"upseeri",        (u"upseer",      u"upser", u"upsier")),
-    (u"upseeristo",     (u"upseeristo",  u"upsieristo")),
+    (u"muhamettilainen", (u"muhamettilai", u"muhammettilai", u"mahomettilai", u"muhamedilai")),
+    (u"musiikki",        (u"musiik",      u"musik")),
+    (u"paneeli",         (u"paneel",      u"panel")),
+    (u"parhaisto",       (u"parhaisto",   u"parahisto")),
+    (u"paronitar",       (u"paronit",     u"paroonit")),
+    (u"pasuuna",         (u"pasuun",      u"pasun")),
+    (u"pataljoona",      (u"pataljoon",   u"pataljon")),
+    (u"patriisi",        (u"patriis",     u"patris")),
+    (u"patruuna",        (u"patruun",     u"patrun")),
+    (u"perimmäinen",     (u"perimmäi",    u"perimäi")),
+    (u"persoona",        (u"persoon",     u"person")),
+    (u"piispa",          (u"piisp",       u"pisp")),
+    (u"pioneeri",        (u"pioneer",     u"pioner")),
+    (u"pioni",           (u"pion",        u"pioon")),
+    (u"plutoona",        (u"plutoon",     u"pluton")),
+    (u"poliisi",         (u"poliis",      u"polis")),
+    (u"poliitikko",      (u"poliitik",    u"politik", u"poliitiik")),
+    (u"poliittinen",     (u"poliitti",    u"politti", u"poliittii")),
+    (u"politiikka",      (u"politiik",    u"politik", u"poliitiik")),
+    (u"Polynesia",       (u"Polynesi",    u"Polyneesi")),
+    (u"positiivi",       (u"positiiv",    u"positiv")),
+    (u"posliini",        (u"posliin",     u"poslin"   u"porsliin", u"porslin")),
+    (u"preettori",       (u"preettor",    u"preetor", u"pretor")),
+    (u"pretoriaani",     (u"pretoriaan",  u"pretorian")),
+    (u"pyramidi",        (u"pyramid",     u"pyramiid")),
+    (u"päällimmäinen",   (u"päällimmäi",  u"päällimäi", u"päälimäi", u"päälimmäi")),
+    (u"päärynä",         ((u"pääryn", u"pääron", u"NimisanaApila_ä", u"NimisanaApila_aä"), )),
+    (u"reettori",        (u"reettor",     u"reetor")),
+    (u"reunimmainen",    (u"reunimmai",   u"reunimai")),
+    (u"romanttinen",     (u"romantti",    u"romanti", u"romantilli")),
+    (u"samanlainen",     (u"samanlai",    u"samallai")),
+    (u"samojedi",        (u"samojed",     u"samojeed")),
+    (u"sampanja",        (u"sampanj",     u"samppanj")),
+    (u"saraseeni",       (u"saraseen",    u"sarasen")),
+    (u"sapatti",         (u"sapat",       u"sabat")),
+    (u"sapeli",          (u"sapel",       u"sapeel")),
+    (u"seminaari",       (u"seminaar",    u"seminar")),
+    (u"senaatti",        ((u"senaat",     u"senaat", u"NimisanaTatti_a", u"NimisanaRisti_a"),
+                          (u"senaat",     u"senat",  u"NimisanaTatti_a", u"NimisanaRisti_a"))),
+    (u"senaattori",      (u"senaattor",   u"senaator")),
+    (u"serafi",          (u"seraf",       u"seraaf")),
+    (u"shampanja",       (u"shampanj",    u"shamppanj")),
+    (u"sihteeri",        (u"sihteer",     u"sihter", u"sihtier")),
+    (u"sitruuna",        (u"sitruun",     u"sitrun", u"sitroon", u"sitron")),
+    (u"siviili",         (u"siviil",      u"sivil")),
+    (u"slaavilainen",    (u"slaavilai",   u"slavilai")),
+    (u"soolo",           (u"soolo",       u"solo")),
+    (u"soopeli",         (u"soopel",      u"sopel", u"soobel")),
+    (u"stationaarinen",  ((u"stationaari", u"stationääri", u"LaatusanaNainenInen_a ", u"LaatusanaNainenInen_ä"), )),
+    (u"synagoga",        (u"synagog",      u"synagoog")),
+    (u"taimmainen",      (u"taimmai",      u"takimai", u"taemmai", u"taaemmai")),
+    (u"teatteri",        (u"teatter",      u"teaater", u"teaatter", u"teater")),
+    (u"temperamentti",   (u"temperament",  u"tempperament")),
+    (u"tooga",           (u"toog",         u"tog")),
+    (u"topaasi",         (u"topaas",       u"topas")),
+    (u"Toscana",         (u"toscan",       u"toskan")),
+    (u"Traakia",         (u"Traaki",       u"Traki")),
+    (u"traakialainen",   (u"traakialai",   u"trakialai")),
+    (u"tussi",           (u"tuss",         u"tush")),
+    (u"tällainen",       ((u"tällai",      u"tällai", u"NimiLaatusanaNainenInen_a", u"NimiLaatusanaNainenInen_ä"),
+                          (u"tällai",      u"tälläi", u"NimiLaatusanaNainenInen_a", u"NimiLaatusanaNainenInen_ä"),
+                          (u"tällai",      u"tälläi", u"NimiLaatusanaNainenInen_a", u"NimiLaatusanaNainenInen_a"))),
+    (u"ulommainen",      (u"ulommai",      u"uloimmai")),
+    (u"upseeri",         (u"upseer",       u"upser", u"upsier")),
+    (u"upseeristo",      (u"upseeristo",   u"upsieristo")),
+    (u"vihkiäinen",      (u"vihkiäi",      u"vihkijäi")),
     (u"yhdeksäinen",        (u"yhdeksäi",         u"yheksäi")),
     (u"yhdeksän",           (u"yhdeks",           u"yheks")),
     (u"yhdeksänkymppinen",  (u"yhdeksänkymppi",   u"yheksänkymppi")),
@@ -395,6 +434,7 @@ word_list = [
     (u"yhdeksänlainen",     (u"yhdeksänlai",      u"yheksänlai")),
     (u"yhdeksännes",        (u"yhdeksänne",       u"yheksänne")),
     (u"yhdeksäs",           (u"yhdeksä",          u"yheksä")),
+    (u"ylimmäinen",         (u"ylimmäi",          u"ylimäi")),
 
     (u"ien",     [u"[Ln][Xp]ien[X]iken[Sn][Ny]e:ikene Loppu ;"]),
 ##                  u"[Ln][Xp]ien[X]ien[Ses][Ny]nä:iennä NimisanaLiOlV_ä ;"
@@ -460,6 +500,7 @@ function_list = [
     (lambda line, word: outfile.write (u"[Ln][Xp]%s[X]%s:%s SukijaLohi ;\n" %
                                        (word, word[0:len(word)-1], word[0:len(word)-1])),
      (u"lohi",
+      u"tyynenmerenlohi",   # On Joukahaisessa.
       u"uuhi")),
 
     # Ahven taipuu kuten sisar, paitsi että yksikön olento on myös ahvenna.
@@ -502,6 +543,20 @@ function_list = [
       u"tiivis")),
 
      (write_altis, (u"altis", )),
+
+     # 11 paistaa (9, 9). Tuomi s. 1, 2, 8, 11, 12, 15, 17.
+     #
+     (write_virkkaa, (u"vilkkaa",
+                      u"virkkaa")),
+     (write_paistaa, (u"paistaa", )),
+     (write_paahtaa, (u"paahtaa",
+                      u"raistaa",
+                      u"saattaa",
+                      u"taittaa",
+                      u"palttaa",
+                      u"varttaa")),
+     (lambda line, word: outfile.write (line.replace (u"NimisanaVati_ä", u"SukijaNeiti")),
+       (u"neiti")),
 ]
 
 def convert_to_dictionary (word_list):
@@ -512,21 +567,6 @@ def convert_to_dictionary (word_list):
 sukija_dictionary = convert_to_dictionary (word_list)
 
 
-def convert_function_list_to_dictionary (function_list):
-    l0 = map (lambda x : x[1], function_list)
-    l1 = map (lambda x : [x[0] for i in range (len(x[1]))], function_list)
-    d1 = dict (zip (l0, l1))
-    d2 = dict()
-    for key, value in d1.iteritems():
-        d2.update (zip (key, value))
-    return d2
-
-
-#function_dictionary = convert_function_list_to_dictionary (function_list)
-#print (function_dictionary)
-#sukija_dictionary.update (function_dictionary)
-
-
 def error (line):
     sys.stderr.write (line)
     sys.stderr.write ("Wrong format in sukija_dictionary.\n")
@@ -547,8 +587,11 @@ def write_tuple (line, key, g):
              replace_and_write (line, g[0], g[i])
      elif type(g[0]) == TupleType:
          for i in range (0, len(g)):
-             s = line.replace (g[i][2], g[i][3])
-             outfile.write (replace (s, g[i][0], g[i][1]))
+             if (len(g[i]) == 2):
+                 replace_and_write (line, g[i][0], g[i][1])
+             else:
+                 s = line.replace (g[i][2], g[i][3])
+                 outfile.write (replace (s, g[i][0], g[i][1]))
      else:
          error (line)
 
@@ -578,69 +621,53 @@ def generate_from_function (r, line, function_list):
             x[0] (line, r.group(1))
 
 
+def generate_xiljoona (line):
+    if (line.startswith ("[Sn]")):
+        u = line.replace (u"miljoona@", u"miljona@")
+        u = u.replace (u"miljoonat@", u"miljonat@")
+        u = u.replace (u"biljoona@",  u"biljona@")
+        u = u.replace (u"biljoonat@", u"biljonat@")
+        u = u.replace (u"triljoona@",  u"triljona@")
+        u = u.replace (u"triljoonat@", u"triljonat@")
+        outfile.write (u)
+    else:
+        outfile.write (line.replace (u"iljoon", u"iljon"))
+        if (line.startswith (u"@") and line.find (u"iljoonien:")):
+            outfile.write (line.replace (u"iljoonien", u"iljoonain"))
+            outfile.write (line.replace (u"iljoonien", u"iljonain"))
+
+
 # Copy Voikko vocabulary and insert forms that Sukija needs.
 #
 while True:
     line = infile.readline()
     if line == u"":
-        break;
+        break
+    if OPTIONS["sukija-ys"]:
+        line = line.replace (u"@P.YS_EI_JATKOA.ON@", u"")
     outfile.write (line)
     if line == u"LEXICON Sanasto\n":
         outfile.write (u"Sukija ;\n")
+    if line == u"LEXICON Joukahainen_p\n":
+        outfile.write (u"SukijaEtuliite ;\n")
+
     generate_from_pattern (line, spelling_pattern_list)
 
-    r = r = base_form_re.search (line)
+    r = base_form_re.search (line)
     if r:
         generate_word (r, line, sukija_dictionary)
         generate_from_function (r, line, function_list)
+    if (re_Xiljoona.search (line)):
+        generate_xiljoona (line)
 infile.close()
 
-
 outfile.write (u"\n\n\n")
 
-OUTPUT = [
-u"LEXICON SukijaNuolaista_a",
-u"s:s   Nuolaista_w_a ;",
-u"s:s   Nuolaista_s_a ;",
-u"s:s   Johdin_U_arvelu_a ;",
-u"LEXICON SukijaNuolaista_ä",
-u"s:s   Nuolaista_w_ä ;",
-u"s:s   Nuolaista_s_ä ;",
-u"s:s   Johdin_U_arvelu_ä ;",
-u"LEXICON SukijaNuolaista_aä",
-u"s:s   Nuolaista_w_a ;",
-u"s:s   Nuolaista_w_ä ;",
-u"s:s   Nuolaista_s_a ;",
-u"s:s   Nuolaista_s_ä ;",
-u"s:s   Johdin_U_arvelu_a ;",
-u"s:s   Johdin_U_arvelu_ä ;",
-u"LEXICON SukijaRangaista_a",
-u"gas:gas Nuolaista_w_a ;",
-u"kas:kas Nuolaista_s_a ;",
-u"kas:kas Johdin_U_arvelu_a ;",
-u"LEXICON SukijaHerttua",
-u"NimisanaApila_m_a ;",
-u"LEXICON SukijaLohi",
-u"[Sg][Nm]t:t NimisanaMonikonGenetiiviEnJatko_a ;",
-u"LEXICON SukijaOmistusliite_a_in",
-u"Omistusliite_a ;",
-u"[O1y]in:in Liitesana_a ;",
-u"LEXICON SukijaTuta",
-u"[Tn1p]takse:takse SukijaOmistusliite_a_in ;",
-u"[Tn2]ten:ten Liitesana_a ;",
+while True:
+    line = sukijafile.readline()
+    if line == u"":
+        break
+    outfile.write (line)
+sukijafile.close()
 
-u"LEXICON Sukija",
-u"[Ln][Xp]aatelisneiti[X][Xr]ppppppp=[X]aatelisnei[Sg][Ny]den:aatelisneiden Loppu ;",
-u"[Ln][Xp]herrasneiti[X][Xr]pppppp=[X]herrasnei[Sg][Ny]den:herrasneiden Loppu ;",
-u"[Ln][Xp]hovineiti[X][Xr]pppp=[X]hovinei[Sg][Ny]den:hovineiden Loppu ;",
-u"[Ln][Xp]neiti[X]nei[Sg][Ny]den:neiden Loppu ;",
-u"[Ln][Xp]nuorimies[X]nuornamiesnä:nuornamiesnä NimisanaLiOlV_ä ;",
-]
-
-OUTPUT.extend (generate (lexicon_kaunis, u"Laatusana", u"a"))
-OUTPUT.extend (generate (lexicon_kaunis, u"Laatusana", u"ä"))
-OUTPUT.extend (generate (lexicon_kaunis, u"NimiLaatusana", u"a"))
-OUTPUT.extend (generate (lexicon_altis,  u"NimiLaatusana", u"a"))
-
-for x in OUTPUT:
-    outfile.write (x + u"\n")
+outfile.close()
